
module MultilevelChainSampler

using AbstractMCMC
using Distributions
using Random
using StatsBase

# reexport overriden functions
export sample, logpdf 

# proposals
export propose 
export StaticProposal, RandomWalk
export TransformedProposal
export StackedProposal, ComponentwiseProposal

# log probability density models
export evaluate
export LogDensity

# hierarchical output 
export subchain
export MultilevelChains

# samplers
export MH 
export MLDA

# implement these functions to support a new sampler
export supports_sampler
export saves_logprob
export saves_reject
export saves_subchain
export has_fixed_length

export autocor

# source files
include("proposal.jl")
include("logdensity.jl")
include("chains.jl")

include("samplers/metropolis_hastings.jl")
include("samplers/delayed_acceptance.jl")


end